#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using System.Collections.Generic;
using NinjaTrader.Gui.Design;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public class DistanceDotUtils {
		public static Rectangle getCanvasRectangle(ChartControl chart, int offset) {
			Rectangle canvasRect = new Rectangle(chart.CanvasLeft-offset,chart.CanvasTop-offset,
												chart.CanvasRight-chart.CanvasLeft+offset,chart.CanvasBottom-chart.CanvasTop+offset);
			return canvasRect;
		}
		
			
		public static void drawExtendedLine(ChartControl chart, Graphics g,Pen pen, double x, double y, double m) {
			
			Rectangle rect = getCanvasRectangle(chart,10);
			if (rect==null) return;
			Point[] points = calculateRectangleLineIntersection(rect,x,y,m);
			if (points==null)return;
			if (points.Length==2) {
				g.DrawLine(pen,points[0].X,points[0].Y,points[1].X,points[1].Y);
			}
		}
		
		
		public static void drawRay(ChartControl chart, Graphics g,Pen pen, double x, double y, double m, int side) {
			
			Rectangle rect = getCanvasRectangle(chart,10);
			if (rect==null) return;
			Point[] points = calculateRectangleLineIntersection(rect,x,y,m);
			if (points==null)return;
			if (points.Length==2) {
				Point pt = new Point(0,0);
				if (side == 1 && points[0].X>=x) {
					pt = (points[0].X>=points[1].X?points[0]:points[1]); 
				}
				else if (side==1 && points[1].X>x) {
					pt = (points[1].X>=points[0].X?points[1]:points[0]);
				}
				else if (side==0 && points[0].X<=x)  {
					pt = (points[0].X<=points[1].X?points[0]:points[1]);
				}
				else if (side==0 && points[1].X<x) {
					pt =  (points[1].X<=points[0].X?points[1]:points[0]);
				} else {
					return;
				}
			    g.DrawLine(pen,(int)x,(int)y,pt.X,pt.Y);
			}
		}
		
		
		
		
		
		private static bool inBounds(Rectangle rect, double x, double y) {
			if (x>=rect.Left && x<=rect.Right && y>=rect.Top && y<=rect.Bottom) {
				return true;
			}
			return false;
		}
		
		public static Point[] calculateRectangleLineIntersection (Rectangle rect, double x1, double y1, double m) {
			double x2 = x1 + 4;
			double y2 = y1 + m*(x2-x1);
			return calculateRectangleLineIntersection(rect,x1,y1,x2,y2);		
		}
		
		public static Point[] calculateRectangleLineIntersection (Rectangle rect, double x1, double y1, double x2, double y2) {
			double m = (y2-y1)/(x2-x1);
			Point[] result = new Point[2];
			int index = 0;
			double  x = 0, y =0;
			//calculate possible 4 points of intersection and determine if it is in the rectangle
			//y-y1 = m*(x-x1)
			
			x = rect.Left; y = m*(x-x1)+y1; 
			if (inBounds(rect,x,y) && (index==0 || (result[0].Y!=y && result[0].X!=x))) {
				result[index] = new Point((int)x,(int)y);
				++index;
			}
			
			x = rect.Right; y = m*(x-x1)+y1;
			if (inBounds(rect,x,y) && (index==0 || (result[0].Y!=y && result[0].X!=x))) {
				result[index] = new Point((int)x,(int)y);
				++index;
			}
			
			if (index==2) return result;
			y = rect.Bottom; x = (y-y1)/m+x1;
			if (inBounds(rect,x,y) && (index==0 || (result[0].Y!=y && result[0].X!=x))) {
				result[index] = new Point((int)x,(int)y);
				++index;
			}
			
			if (index==2) return result;
			y = rect.Top; x = (y-y1)/m+x1;
			if (inBounds(rect,x,y) && (index==0 || (result[0].Y!=y && result[0].X!=x))) {
				result[index] = new Point((int)x,(int)y);
				++index; 
			}		
			
			if (index==2) return result;
			return null;

		}
		

	
	}


    [Description("Enter the description of your new custom indicator here")]
    public class DistanceDot : Indicator
    {
        #region Variables
        // Wizard generated variables
            public int span = 20; // Default setting for Span
		    public Color taxicolor = Color.DarkGray;
			public Color averagecolor = Color.Red;
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
		
		ZigZagUTG zig;
		//Fractals fractal;
        protected override void Initialize()
        {
	
            Overlay				= true;
			CalculateOnBarClose = false;
			zig = ZigZagUTG(false,span,1);
			//fractal = Fractals();
			zig.Update();
			//fractal.Update();
        }
		

		public int barIdx(int n) {
			return Bars.Count-1-n;
		}
		
		public int median3 (int a,int b, int c) {
			int[] items = {a,b,c};
			Array.Sort(items,0,3);
			return items[1];
			
		}
		
		
		public void drawDot(Graphics g, Pen pen, Brush brush, double x, double y, double radius) {
			g.DrawEllipse(pen,(int)(x-radius),(int)(y-radius),(int)(radius*2),(int)(radius*2));
			g.FillEllipse(brush,(int)(x-radius),(int)(y-radius),(int)(radius*2),(int)(radius*2));	
		}

		public void drawDot(Graphics g, Pen pen, double x, double y, double radius) {
			drawDot(g,pen,pen.Brush,x,y,radius);
		}
		
	
		public override void Plot (Graphics graphics, Rectangle bounds, double min, double max) {
			graphics.SmoothingMode = SmoothingMode.AntiAlias;
			Pen averagePen = new Pen(new SolidBrush(averagecolor));
			Pen taxiPen = new Pen(new SolidBrush(taxicolor));
Pen taxiPen2 = new Pen(new SolidBrush(Color.DarkRed));

			try {
				zig.Update();
				List<int> pivotIndex = new List<int>();
				for (int i=0;i<Math.Min(Bars.Count,zig.Pivot.Count); i++) {
					if (zig.Pivot.ContainsValue(i)) {
						pivotIndex.Add(i);
					}
				}
			
				//draw dots for every 3 points
				for (int i=0;i<=pivotIndex.Count-3;i++) {
					
					int i1 = pivotIndex[i];
					int i2 = pivotIndex[i+1];
					int i3 = pivotIndex[i+2];
					
					/*
					//calculate minimum absolute deviation of fractals from i1 to i3
					int count = 0;
					int median = 0;
					List<double> y_values = new List<double>();
					for (int j=i1;j<=i3;j++) {
						
						if (fractal.Plot0.ContainsValue(j) || fractal.Plot1.ContainsValue(j)) {
							++count;
							if (fractal.Plot0.ContainsValue(j)) {
								y_values.Add(fractal.Plot0[j]);
							}
							if (fractal.Plot1.ContainsValue(j)) {
								y_values.Add(fractal.Plot1[j]);
							}
						}
					}
					
					if (count%2==1)
						median = (count+1)/2;
					else
						median = count/2;
					
					y_values.Sort();
					int offset = 0; //this is saying  - lets add one point in the direction of the next swing, in order to bias it that way ? 
					double median_y = 0;
					if (y_values.Count%2==1) {
						int index = (y_values.Count+1)/2-1+offset;
						if (index>=y_values.Count) return;
						median_y = y_values[index];
					} else {
						count = 0;
						int index = 0;
						for (int j=i1;j<=i3;j++) {
							
							if (fractal.Plot0.ContainsValue(j) || fractal.Plot1.ContainsValue(j)) {
								++count;
								if (count==median) {
									if (fractal.Plot1.ContainsValue(j)) {
										index = (y_values.Count)/2-1- offset;
									} else {
										index = (y_values.Count)/2-1+ offset;
									}
									break;
								}
							}
						}
							
						if (index==0) return;
						if (index>=y_values.Count) return;
						median_y = y_values[index];
					}
					
					count=0;
					//get the time value of the median , searching from right to left
					DateTime time = DateTime.Now; 
					bool got_time = false;
					for (int j=i1;j<=i3;j++) {
						if (fractal.Plot0.ContainsValue(j) || fractal.Plot1.ContainsValue(j)) {
							count+=1;
							if (count==median) {
								time = fractal.Time[j];
								got_time=true;
								break;
							}
						}	
					}
					if (got_time==false) return;
			*/
					int x1 = ChartControl.GetXByTime(zig.Time[i1]);
					int y1 = ChartControl.GetYByValue(this,zig.Pivot[i1]);
										
					int x2 = ChartControl.GetXByTime(zig.Time[i2]);
					int y2 = ChartControl.GetYByValue(this,zig.Pivot[i2]);
					
					int x3 = ChartControl.GetXByTime(zig.Time[i3]);
					int y3 = ChartControl.GetYByValue(this,zig.Pivot[i3]);
					
					double x = (x1+x2*1.0)/2.0;//ChartControl.GetXByTime(time);
					double y = (y2+1.0*y3)/2.0;//ChartControl.GetYByValue(this,median_y);
									
					int a = median3(x1,x2,x3);
					int b = median3(y1,y2,y3);
					
					int radius = 1;
					drawDot(graphics,taxiPen,a,b,1); 
					if (x-x1!=0) {
						double m1 = (y*1.0-y1)/(x*1.0-x1);
						if (m1!=0) {
							double l = 25;
							
							double m = -1/m1;
							double delta_x = l/Math.Sqrt(1+m*m);			
							double delta_y = delta_x*m; 
							drawDot(graphics,taxiPen,a+delta_x,b+delta_y,radius);
							delta_x = -l/Math.Sqrt(1+m*m);
							delta_y = delta_x*m;
							drawDot(graphics,taxiPen,a+delta_x,b+delta_y,radius);
							
							m = m1;
						    delta_x = l/Math.Sqrt(1+m*m);
							delta_y = delta_x*m;
							drawDot(graphics,taxiPen,a+delta_x,b+delta_y,radius);
							delta_x = -l/Math.Sqrt(1+m*m);
							delta_y = delta_x*m;
							drawDot(graphics,taxiPen,a+delta_x,b+delta_y,radius);
							
						}
						else {
							drawDot(graphics,taxiPen,a,b-50,radius);
							
						}
					}	
	
					drawDot(graphics,averagePen,x,y,3);
					
					//drawDot(graphics,taxiPen2,(x2+x3*1.0)/2.0,(y1+y2*1.0)/2.0,radius);
				}
			} catch (Exception e) {
				Print(e.ToString());
			}
		}
		

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			zig.Update();
			//fractal.Update();
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Plot0
        {
            get { return Values[0]; }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int Span
        {
            get { return span; }
            set { span = Math.Max(1, value); }
        }
		
		
		
		
		
		[Browsable(false)]
    	public string TaxiColorSerialize
    	{
        		get { return SerializableColor.ToString(taxicolor); }
        		set { taxicolor = SerializableColor.FromString(value); }
    	}
		
		[XmlIgnore()]
		[Description("'Taxi Point' Color")]
		[GridCategory("Parameters")]
		public Color TaxiColor
		{
			get { return taxicolor; }
			set { taxicolor = value; }
		}
		
		[Browsable(false)]
    	public string AverageColorSerializable
    	{
        		get { return SerializableColor.ToString(this.AverageColor); }
        		set { this.AverageColor = SerializableColor.FromString(value); }
    	}
		
				[XmlIgnore()]
		[Description("'Average Point' Color")]
		[GridCategory("Parameters")]
		public Color AverageColor
		{
			get { return averagecolor; }
			set { averagecolor = value; }
		}
		
		
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private DistanceDot[] cacheDistanceDot = null;

        private static DistanceDot checkDistanceDot = new DistanceDot();

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public DistanceDot DistanceDot(Color averageColor, int span, Color taxiColor)
        {
            return DistanceDot(Input, averageColor, span, taxiColor);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public DistanceDot DistanceDot(Data.IDataSeries input, Color averageColor, int span, Color taxiColor)
        {
            if (cacheDistanceDot != null)
                for (int idx = 0; idx < cacheDistanceDot.Length; idx++)
                    if (cacheDistanceDot[idx].AverageColor == averageColor && cacheDistanceDot[idx].Span == span && cacheDistanceDot[idx].TaxiColor == taxiColor && cacheDistanceDot[idx].EqualsInput(input))
                        return cacheDistanceDot[idx];

            lock (checkDistanceDot)
            {
                checkDistanceDot.AverageColor = averageColor;
                averageColor = checkDistanceDot.AverageColor;
                checkDistanceDot.Span = span;
                span = checkDistanceDot.Span;
                checkDistanceDot.TaxiColor = taxiColor;
                taxiColor = checkDistanceDot.TaxiColor;

                if (cacheDistanceDot != null)
                    for (int idx = 0; idx < cacheDistanceDot.Length; idx++)
                        if (cacheDistanceDot[idx].AverageColor == averageColor && cacheDistanceDot[idx].Span == span && cacheDistanceDot[idx].TaxiColor == taxiColor && cacheDistanceDot[idx].EqualsInput(input))
                            return cacheDistanceDot[idx];

                DistanceDot indicator = new DistanceDot();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.AverageColor = averageColor;
                indicator.Span = span;
                indicator.TaxiColor = taxiColor;
                Indicators.Add(indicator);
                indicator.SetUp();

                DistanceDot[] tmp = new DistanceDot[cacheDistanceDot == null ? 1 : cacheDistanceDot.Length + 1];
                if (cacheDistanceDot != null)
                    cacheDistanceDot.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheDistanceDot = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DistanceDot DistanceDot(Color averageColor, int span, Color taxiColor)
        {
            return _indicator.DistanceDot(Input, averageColor, span, taxiColor);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.DistanceDot DistanceDot(Data.IDataSeries input, Color averageColor, int span, Color taxiColor)
        {
            return _indicator.DistanceDot(input, averageColor, span, taxiColor);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.DistanceDot DistanceDot(Color averageColor, int span, Color taxiColor)
        {
            return _indicator.DistanceDot(Input, averageColor, span, taxiColor);
        }

        /// <summary>
        /// Enter the description of your new custom indicator here
        /// </summary>
        /// <returns></returns>
        public Indicator.DistanceDot DistanceDot(Data.IDataSeries input, Color averageColor, int span, Color taxiColor)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.DistanceDot(input, averageColor, span, taxiColor);
        }
    }
}
#endregion
